@@ -20,56 +20,30 @@
  *         Mattias Bengtsson <mattias.jc.bengtsson@gmail.com>
  */
 
-const Cairo = imports.cairo;
-const Gdk = imports.gi.Gdk;
-const GObject = imports.gi.GObject;
-const Gtk = imports.gi.Gtk;
-const Mainloop = imports.mainloop;
+import Cairo from 'cairo';
+import Gdk from 'gi://Gdk';
+import GLib from 'gi://GLib';
+import GObject from 'gi://GObject';
+import Gtk from 'gi://Gtk';
 
-const Application = imports.application;
-const InstructionRow = imports.instructionRow;
-const PlaceStore  = imports.placeStore;
-const RouteEntry = imports.routeEntry;
-const RouteQuery = imports.routeQuery;
-const StoredRoute = imports.storedRoute;
-const TransitArrivalRow = imports.transitArrivalRow;
-const TransitItineraryRow = imports.transitItineraryRow;
-const TransitLegRow = imports.transitLegRow;
-const TransitMoreRow = imports.transitMoreRow;
-const TransitOptionsPanel = imports.transitOptionsPanel;
-const Utils = imports.utils;
+import {Application} from './application.js';
+import {InstructionRow} from './instructionRow.js';
+import {PlaceStore} from './placeStore.js';
+import {QueryPoint} from './routeQuery.js';
+import {RouteEntry} from './routeEntry.js';
+import {RouteQuery} from './routeQuery.js';
+import {StoredRoute} from './storedRoute.js';
+import {TransitArrivalRow} from './transitArrivalRow.js';
+import {TransitItineraryRow} from './transitItineraryRow.js';
+import {TransitLegRow} from './transitLegRow.js';
+import {TransitMoreRow} from './transitMoreRow.js';
+import {TransitOptionsPanel} from './transitOptionsPanel.js';
+import * as Utils from './utils.js';
 
-var Sidebar = GObject.registerClass({
-    Template: 'resource:///org/gnome/Maps/ui/sidebar.ui',
-    InternalChildren: [ 'distanceInfo',
-                        'entryList',
-                        'instructionList',
-                        'instructionWindow',
-                        'instructionSpinner',
-                        'instructionStack',
-                        'errorLabel',
-                        'modeBikeToggle',
-                        'modeCarToggle',
-                        'modePedestrianToggle',
-                        'modeTransitToggle',
-                        'timeInfo',
-                        'linkButtonStack',
-                        'transitWindow',
-                        'transitRevealer',
-                        //'transitOptionsPanel',
-                        'transitHeader',
-                        'transitListStack',
-                        'transitOverviewListBox',
-                        'transitItineraryHeader',
-                        'transitItineraryListBox',
-                        'transitItineraryBackButton',
-                        'transitItineraryTimeLabel',
-                        'transitItineraryDurationLabel',
-                        'transitAttributionLabel']
-}, class Sidebar extends Gtk.Revealer {
+export class Sidebar extends Gtk.Revealer {
 
-    _init(mapView) {
-        super._init({ transition_type: Gtk.RevealerTransitionType.SLIDE_LEFT });
+    constructor(mapView) {
+        super({ transition_type: Gtk.RevealerTransitionType.SLIDE_LEFT });
 
         this._mapView = mapView;
 
@@ -81,8 +55,7 @@ var Sidebar = GObject.registerClass({
          * itinerary header widget into the GtkStack to get the correct
          * animation direction.
          */
-        this._transitOptionsPanel =
-            new TransitOptionsPanel.TransitOptionsPanel({ visible: true });
+        this._transitOptionsPanel = new TransitOptionsPanel({ visible: true });
         this._transitHeader.add_named(this._transitOptionsPanel, 'options');
         this._transitHeader.add_named(this._transitItineraryHeader,
                                       'itinerary-header');
@@ -157,18 +130,21 @@ var Sidebar = GObject.registerClass({
     }
 
     _initQuerySignals() {
+        this._numRouteEntries = 0;
         this._query.connect('point-added', (obj, point, index) => {
             this._createRouteEntry(index, point);
+            this._numRouteEntries++;
         });
 
         this._query.connect('point-removed', (obj, point, index) => {
             let row = this._entryList.get_row_at_index(index);
-            row.destroy();
+            this._entryList.remove(row);
+            this._numRouteEntries--;
         });
     }
 
     _cancelStore() {
-        Mainloop.source_remove(this._storeRouteTimeoutId);
+        GLib.source_remove(this._storeRouteTimeoutId);
         this._storeRouteTimeoutId = 0;
     }
 
@@ -176,17 +152,18 @@ var Sidebar = GObject.registerClass({
         let type;
         if (index === 0)
             type = RouteEntry.Type.FROM;
-        else if (index === this._entryList.get_children().length)
+        else if (index === this._numRouteEntries)
             type = RouteEntry.Type.TO;
         else
             type = RouteEntry.Type.VIA;
 
-        let routeEntry = new RouteEntry.RouteEntry({ type: type,
-                                                     point: point,
-                                                     mapView: this._mapView });
+        let routeEntry = new RouteEntry({ type: type,
+                                          point: point,
+                                          mapView: this._mapView });
 
         // add handler overriding tab focus behavior on route entries
-        routeEntry.entry.connect('focus', this._onRouteEntryFocus.bind(this));
+        // TODO: how to set up tab handling using GTK4?
+        //routeEntry.entry.connect('focus', this._onRouteEntryFocus.bind(this));
         // add handler for
         routeEntry.entry.connect('notify::place', () => {
             this._onRouteEntrySelectedPlace(routeEntry.entry);
@@ -195,7 +172,7 @@ var Sidebar = GObject.registerClass({
 
         if (type === RouteEntry.Type.FROM) {
             routeEntry.button.connect('clicked', () => {
-                let lastIndex = this._entryList.get_children().length;
+                let lastIndex = this._numRouteEntries;
                 this._query.addPoint(lastIndex - 1);
                 // focus on the newly added point's entry
                 this._entryList.get_row_at_index(lastIndex - 1).get_child().entry.grab_focus();
@@ -203,7 +180,7 @@ var Sidebar = GObject.registerClass({
 
             this.connect('notify::child-revealed', () => {
                 if (this.child_revealed)
-                    routeEntry.entry.grab_focus_without_selecting();
+                    routeEntry.entry.grab_focus();
             });
         } else if (type === RouteEntry.Type.VIA) {
             routeEntry.button.connect('clicked', () => {
@@ -244,12 +221,12 @@ var Sidebar = GObject.registerClass({
     }
 
     _onRouteEntrySelectedPlace(entry) {
-        let index = this._getIndexForRouteEntry(entry);
+        let [index, numEntries] = this._getIndexForRouteEntryAndNumEntries(entry);
 
         /* if a new place is selected and it's not the last entry, focus next
          * entry
          */
-        if (entry.place && index < this._entryList.get_children().length - 1) {
+        if (entry.place && index < numEntries - 1) {
             let nextPlaceEntry =
                 this._entryList.get_row_at_index(index + 1).get_child().entry;
 
@@ -258,15 +235,27 @@ var Sidebar = GObject.registerClass({
         }
     }
 
-    _getIndexForRouteEntry(entry) {
-        for (let i = 0; i < this._entryList.get_children().length; i++) {
-            let routeEntry = this._entryList.get_row_at_index(i).get_child();
+    _getIndexForRouteEntryAndNumEntries(entry) {
+        let index = 0;
+        let foundIndex = -1;
+
+        for (let item of this._entryList) {
+            let routeEntry = item.get_child();
 
             if (routeEntry.entry === entry)
-                return i;
+                foundIndex = index;
+
+            index++;
         }
 
-        return -1;
+        return [foundIndex, index];
+    }
+
+    // this is needed to be called on shutdown to avoid a GTK warning
+    unparentSearchPopovers() {
+        for (let item of this._entryList) {
+            item.get_child().entry.popover.unparent();
+        }
     }
 
     _initInstructionList() {
@@ -294,8 +283,13 @@ var Sidebar = GObject.registerClass({
 
         transitPlan.connect('no-more-results', () => {
             // set the "load more" row to indicate no more results
-            let numRows = this._transitOverviewListBox.get_children().length;
-            let loadMoreRow = this._transitOverviewListBox.get_row_at_index(numRows - 2);
+            let loadMoreRow;
+
+            for (let row of this._transitOverviewListBox) {
+                if (row instanceof TransitMoreRow)
+                    loadMoreRow = row;
+            }
+
             loadMoreRow.showNoMore();
         });
 
@@ -326,12 +320,12 @@ var Sidebar = GObject.registerClass({
             if (this._storeRouteTimeoutId)
                 this._cancelStore();
 
-            this._storeRouteTimeoutId = Mainloop.timeout_add(5000, () => {
+            this._storeRouteTimeoutId = GLib.timeout_add(null, 5000, () => {
                 let placeStore = Application.placeStore;
                 let places = this._query.filledPoints.map(function(point) {
                     return point.place;
                 });
-                let storedRoute = new StoredRoute.StoredRoute({
+                let storedRoute = new StoredRoute({
                     transportation: this._query.transportation,
                     route: route,
                     places: places,
@@ -346,8 +340,8 @@ var Sidebar = GObject.registerClass({
             });
 
             route.turnPoints.forEach((turnPoint) => {
-                let row = new InstructionRow.InstructionRow({ visible: true,
-                                                              turnPoint: turnPoint });
+                let row = new InstructionRow({ visible: true,
+                                               turnPoint: turnPoint });
                 this._instructionList.insert(row, -1);
             });
 
@@ -389,10 +383,23 @@ var Sidebar = GObject.registerClass({
         this._errorLabel.label = msg;
     }
 
+    _clearListBox(listBox) {
+        let rows = [];
+
+        for (let row of listBox) {
+            if (row instanceof Gtk.ListBoxRow)
+                rows.push(row);
+        }
+
+        for (let row of rows) {
+            listBox.remove(row);
+        }
+    }
+
     _clearTransitOverview() {
         let listBox = this._transitOverviewListBox;
-        listBox.forall(listBox.remove.bind(listBox));
 
+        this._clearListBox(listBox);
         this._instructionStack.visible_child = this._transitWindow;
         this._timeInfo.label = '';
         this._distanceInfo.label = '';
@@ -400,7 +407,8 @@ var Sidebar = GObject.registerClass({
 
     _clearTransitItinerary() {
         let listBox = this._transitItineraryListBox;
-        listBox.forall(listBox.remove.bind(listBox));
+
+        this._clearListBox(listBox);
     }
 
     _updateTransitAttribution() {
@@ -438,14 +446,13 @@ var Sidebar = GObject.registerClass({
         let plan = Application.routingDelegator.transitRouter.plan;
 
         plan.itineraries.forEach((itinerary) => {
-            let row =
-                new TransitItineraryRow.TransitItineraryRow({ visible: true,
-                                                              itinerary: itinerary });
+            let row = new TransitItineraryRow({ visible: true,
+                                                itinerary: itinerary });
             this._transitOverviewListBox.insert(row, -1);
         });
         /* add the "load more" row */
-        this._transitOverviewListBox.insert(
-            new TransitMoreRow.TransitMoreRow({ visible: true }), -1);
+        this._transitOverviewListBox.insert(new TransitMoreRow({ visible: true }),
+                                            -1);
 
         /* add an empty list row to get a final separator */
         this._transitOverviewListBox.insert(new Gtk.ListBoxRow({ visible: true }),
@@ -483,24 +490,23 @@ var Sidebar = GObject.registerClass({
         this._clearTransitItinerary();
         for (let i = 0; i < itinerary.legs.length; i++) {
             let leg = itinerary.legs[i];
-            let row = new TransitLegRow.TransitLegRow({ leg: leg,
-                                                        start: i === 0,
-                                                        mapView: this._mapView });
+            let row = new TransitLegRow({ leg: leg,
+                                          start: i === 0,
+                                          mapView: this._mapView });
             this._transitItineraryListBox.insert(row, -1);
         }
 
         /* insert the additional arrival row, showing the arrival place and time */
         this._transitItineraryListBox.insert(
-            new TransitArrivalRow.TransitArrivalRow({ itinerary: itinerary,
-                                                      mapView: this._mapView }),
+            new TransitArrivalRow({ itinerary: itinerary, mapView: this._mapView }),
             -1);
     }
 
 
     _clearInstructions() {
         let listBox = this._instructionList;
-        listBox.forall(listBox.remove.bind(listBox));
 
+        this._clearListBox(listBox);
         this._instructionStack.visible_child = this._instructionWindow;
         this._timeInfo.label = '';
         this._distanceInfo.label = '';
@@ -509,7 +515,7 @@ var Sidebar = GObject.registerClass({
     // Iterate over points and establish the new order of places
     _reorderRoutePoints(srcIndex, destIndex) {
         let points = this._query.points;
-        let srcPlace = this._draggedPoint.place;
+        let srcPlace = this._query.points[srcIndex].place;
 
         // Determine if we are swapping from "above" or "below"
         let step = (srcIndex < destIndex) ? -1 : 1;
@@ -545,86 +551,87 @@ var Sidebar = GObject.registerClass({
         this._query.thaw_notify();
     }
 
-    _onDragDrop(row, context, x, y, time) {
-        let srcIndex = this._query.points.indexOf(this._draggedPoint);
+    _onDragDrop(row, point) {
+        let srcIndex = this._query.points.indexOf(point);
         let destIndex = row.get_index();
 
         this._reorderRoutePoints(srcIndex, destIndex);
-        Gtk.drag_finish(context, true, false, time);
-        return true;
-    }
 
-    _dragHighlightRow(row) {
-        row.opacity = 0.6;
-    }
-
-    _dragUnhighlightRow(row) {
-        row.opacity = 1.0;
-    }
-
-    // Set the opacity of the row we are currently dragging above
-    // to semi transparent.
-    _onDragMotion(row, context, x, y, time) {
-        let routeEntry = row.get_child();
-
-        if (this._draggedPoint && this._draggedPoint !== routeEntry.point) {
-            this._dragHighlightRow(row);
-            Gdk.drag_status(context, Gdk.DragAction.MOVE, time);
-        } else
-            Gdk.drag_status(context, 0, time);
         return true;
     }
 
     // Drag ends, show the dragged row again.
-    _onDragEnd(context, row) {
-        this._draggedPoint = null;
-
-        // Restore to natural height
-        row.height_request = -1;
-        row.get_child().show();
+    _onDragEnd(row) {
+        row.opacity = 1.0;
     }
 
-    // Drag begins, set the correct drag icon and hide the dragged row.
-    _onDragBegin(context, row) {
+    _onDragPrepare(point, source, x, y) {
+        return Gdk.ContentProvider.new_for_value(point);
+    }
+
+    // Drag begins, set the correct drag icon and dim the dragged row.
+    _onDragBegin(source, row) {
         let routeEntry = row.get_child();
-        let width = row.get_allocated_width();
-        let height = row.get_allocated_height();
-        let surface = new Cairo.ImageSurface(Cairo.Format.ARGB32, width, height);
-        let cr = new Cairo.Context(surface)
+        let {x, y, width, height} = row.get_allocation();
+        let paintable = new Gtk.WidgetPaintable({ widget: routeEntry });
 
-        row.draw(cr);
-        this._draggedPoint = routeEntry.point;
-
-        // Set a fixed height on the row to prevent the sidebar height
-        // to shrink while dragging a row.
-        row.height_request = height;
-        row.get_child().hide();
-
-        Gtk.drag_set_icon_surface(context, surface);
+        source.set_icon(paintable, 0, 0);
+        row.opacity = 0.6;
     }
 
     // Set up drag and drop between RouteEntrys. The drag source is from a
     // GtkEventBox that contains the start/end icon next in the entry. And
     // the drag destination is the ListBox row.
     _initRouteDragAndDrop(routeEntry) {
-        let dragIcon = routeEntry.iconEventBox;
+        let dragIcon = routeEntry.icon;
         let row = routeEntry.get_parent();
+        let dragSource = new Gtk.DragSource();
 
-        dragIcon.drag_source_set(Gdk.ModifierType.BUTTON1_MASK,
-                                 null,
-                                 Gdk.DragAction.MOVE);
-        dragIcon.drag_source_add_image_targets();
+        dragIcon.add_controller(dragSource);
 
-        row.drag_dest_set(Gtk.DestDefaults.MOTION,
-                          null,
-                          Gdk.DragAction.MOVE);
-        row.drag_dest_add_image_targets();
+        dragSource.connect('prepare',
+                           this._onDragPrepare.bind(this, routeEntry.point));
+        dragSource.connect('drag-begin',
+                           (source, drag, widget) =>
+                           this._onDragBegin(source, row));
+        dragSource.connect('drag-end',
+                           (source, dele, data) => this._onDragEnd(row));
 
-        dragIcon.connect('drag-begin', (icon, context) => this._onDragBegin(context, row));
-        dragIcon.connect('drag-end', (icon, context) => this._onDragEnd(context, row));
+        let dropTarget = Gtk.DropTarget.new(QueryPoint, Gdk.DragAction.COPY);
 
-        row.connect('drag-leave', this._dragUnhighlightRow.bind(this, row));
-        row.connect('drag-motion', this._onDragMotion.bind(this));
-        row.connect('drag-drop', this._onDragDrop.bind(this));
+        row.add_controller(dropTarget);
+
+        dropTarget.connect('drop',
+                           (target, value, x, y, data) =>
+                           this._onDragDrop(row, value));
     }
-});
+}
+
+GObject.registerClass({
+    Template: 'resource:///org/gnome/Maps/ui/sidebar.ui',
+    InternalChildren: [ 'distanceInfo',
+                        'entryList',
+                        'instructionList',
+                        'instructionWindow',
+                        'instructionSpinner',
+                        'instructionStack',
+                        'errorLabel',
+                        'modeBikeToggle',
+                        'modeCarToggle',
+                        'modePedestrianToggle',
+                        'modeTransitToggle',
+                        'timeInfo',
+                        'linkButtonStack',
+                        'transitWindow',
+                        'transitRevealer',
+                        //'transitOptionsPanel',
+                        'transitHeader',
+                        'transitListStack',
+                        'transitOverviewListBox',
+                        'transitItineraryHeader',
+                        'transitItineraryListBox',
+                        'transitItineraryBackButton',
+                        'transitItineraryTimeLabel',
+                        'transitItineraryDurationLabel',
+                        'transitAttributionLabel']
+}, Sidebar);
