@@ -19,14 +19,15 @@
  * Author: Marcus Lundblad <ml@update.uu.se>
  */
 
-const Cairo = imports.cairo;
-const GLib = imports.gi.GLib;
-const GObject = imports.gi.GObject;
-const Gtk = imports.gi.Gtk;
+import Cairo from 'cairo';
+import GLib from 'gi://GLib';
+import GObject from 'gi://GObject';
+import Graphene from 'gi://Graphene';
+import Gtk from 'gi://Gtk';
 
-const Color = imports.color;
-const Gfx = imports.gfx;
-const Utils = imports.utils;
+import * as Color from './color.js';
+import * as Gfx from './gfx.js';
+import * as Utils from './utils.js';
 
 /* threashhold for route color luminance when we consider it more or less
  * as white, and draw an outline around the label
@@ -41,20 +42,17 @@ const DARK_OUTLINE_LUMINANCE_THREASHHOLD = 0.1;
 const HIGH_CONTRAST_COLOR = '000000';
 const HIGH_CONTRAST_TEXT_COLOR = 'ffffff';
 
-var TransitRouteLabel = GObject.registerClass({
-    Template: 'resource:///org/gnome/Maps/ui/transit-route-label.ui',
-}, class TransitRouteLabel extends Gtk.Label {
+export class TransitRouteLabel extends Gtk.Label {
 
-    _init(params) {
+    constructor(params) {
         let leg = params.leg;
         let compact = params.compact;
 
         delete params.leg;
         delete params.compact;
-        super._init(params);
+        super(params);
 
         this._setLabel(leg, compact);
-        this.connect('draw', this._onDraw.bind(this));
     }
 
     _setLabel(leg, compact) {
@@ -105,20 +103,30 @@ var TransitRouteLabel = GObject.registerClass({
         this.label = '<span foreground="#%s">%s</span>'.format(
                                         textColor,
                                         GLib.markup_escape_text(label, -1));
+
+        this.queue_draw();
     }
 
     /* I didn't find any easy/obvious way to override widget background color
      * and getting rounded corner just using CSS styles, so doing a custom
      * Cairo drawing of a "roundrect"
      */
-    _onDraw(widget, cr) {
-        let width = widget.get_allocated_width();
-        let height = widget.get_allocated_height();
+    vfunc_snapshot(snapshot) {
+        let {x, y, width, height} = this.get_allocation();
+        let rect = new Graphene.Rect();
 
+        rect.init(0, 0, width, height);
+
+        let cr = snapshot.append_cairo(rect);
+
+        // clip off the badge, this seems to avoid some extra space at right/bottom with the CSS
         Gfx.drawColoredBagde(cr, this._color,
                              this._hasOutline ? this._textColor : null,
-                             0, 0, width, height);
-
-        return false;
+                             0, 0, width - 3, height - 3);
+        super.vfunc_snapshot(snapshot);
     }
-});
+}
+
+GObject.registerClass({
+    Template: 'resource:///org/gnome/Maps/ui/transit-route-label.ui',
+}, TransitRouteLabel);
